function [train_feature_cell, train_label_cell, test_dataseq_cell, test_timeseq_cell, test_time_c, test_y_c] = readPHM2015Data_global_traintestseq_csegment_feature(files, segment_length, bins, nfeatures) 
%%  read data and separate the complete data into training data and testing data
%   input:
%       files:          array of plant#
%       segment_length: array of numbers of time pieces, each time segment
%                       is composed of a number of pieces
%       bins:           cell array of bins used for drawing histograms, the
%                       number of cells is the number of dimensions of the
%                       raw data,generated by bin_generator.m
%       nfeatures:      total number of features, generated by
%                       bin_generator.m
%   output:
%       train_feature_cell: cell array of training data, features are 
%                           already extracted, each cell is aplant
%       train_label_cell:   cell array of training fault labels, each cell
%                           is a plant
%       test_dataseq_cell:  cell array of raw testing data, features are
%                           not extracted, each cell is a plant
%       test_time_c:        time of fault events of test data, each cell is
%                           a plant
%       test_y_c:           label of fault events of test data, each cell
%                           is a plant

% prefix of file names
filename_pre = '../PHMtrain/plant_';

% default settings:
% files = [1, 3:15, 17, 18, 21, 23, 24, 25, 27:36, 39, 40];
% fault_index_removed = zeros(1, 60);
% fault_index_removed([41:43, 45:49, 53, 54, 56:60]) = [1023, 1882, 588, 811, 520, 913, 5794, 1429, 879, 420, 906, 1145, 1796, 1922, 1715];

% default length of time pieces: 15 min
time_piece = 0.0104;

nfiles = length(files);

train_feature_cell = cell(nfiles, 1);
train_label_cell = cell(nfiles, 1);
test_dataseq_cell = cell(nfiles, 1);
test_timeseq_cell = cell(nfiles, 1);

test_time_c = cell(nfiles, 1);
test_y_c = cell(nfiles, 1);

for i = 1:length(files)
    
    % read file a
    filename = [filename_pre, num2str(files(i)), 'a', '.csv'];
    fid = fopen(filename);
    dataa = textscan(fid, '%d%s%d%d%d%d%d%d%d%d','delimiter',',');
    fclose(fid);
    % read file c
    filename = [filename_pre, num2str(files(i)), 'c', '.csv'];
    fid = fopen(filename);
    datac = textscan(fid, '%s%s%d','delimiter',',');
    fclose(fid);
        
    tmp_timea_origin = readtime(dataa{2});
    feature_a_origin = double(cell2mat(dataa(3:end)));
    
    %% construct training data
    
    tmp_timea = tmp_timea_origin;
    feature_a = feature_a_origin;
    y_c = datac{3};
    time_c1 = readtime(datac{1});
    y_c_target_list = find(y_c~=6);
    % separate data
    fault_index_removed = y_c_target_list(round(length(y_c_target_list)/2));
    time_c1 = time_c1(1:(fault_index_removed-1));
    time_c2 = readtime(datac{2});
    time_c2 = time_c2(1:(fault_index_removed-1));
    y_c = y_c(1:(fault_index_removed-1));
    time_c = [time_c1, time_c2];
    ncomps = length(unique(dataa{1}));
    dataa_preserved = tmp_timea < max(time_c(:, 2));
    tmp_timea = tmp_timea(dataa_preserved);
    feature_a = feature_a(dataa_preserved, :);
    
    
    % for training, find all the time segments according to fault events 
    target_y_i = find(y_c ~= 6);
    cur_segment = cell(length(target_y_i)+1, 2);
    total_samples = 0;
    for j = 1:length(target_y_i)
        cur_segment{j, 1} = feature_a(tmp_timea>=time_c(target_y_i(j), 1)&tmp_timea<=time_c(target_y_i(j), 2), :);
        total_samples = total_samples + size(cur_segment{j, 1}, 1);
        cur_segment{j, 2} = y_c(target_y_i(j));
    end
    % labels of each fault event
    tmp_y = zeros(length(tmp_timea), 6);
    for j = 1:length(tmp_timea)
            tmp_y(j, y_c(tmp_timea(j)>=time_c(:, 1) & tmp_timea(j)<=time_c(:, 2))) = 1;
    end
    outlier_a_i = find(sum(tmp_y(:, 1:5), 2) == 0);
%     segment_size_max = max([ceil(total_samples / length(target_y_i)), 4]);

    outlier_step_idx = 1;
    
    % outlier: fault label is 6
    tmp_timea = tmp_timea(outlier_a_i, :);
    feature_a = feature_a(outlier_a_i, :);
    j = 1;
    
    % randomly sampling normal events
    % approximately 1200 normal events
    c = sum((segment_length.^2+1))*length(tmp_timea)/ncomps/1200;
%     c = sum((segment_length.^2+1))*length(tmp_timea)/ncomps/(length(target_y_i)*8);
    while j <= length(tmp_timea)
        jj = j;
        next_j = j+1;
        cur_segment_length = segment_length;
        if segment_length(1) ~=0
            cur_segment_length = [0, cur_segment_length];
        end
        for k = cur_segment_length
            time_threshold = time_piece * k + 0.0014;
            accept_ratio = (k^2+1)/c;
            prev_jj = jj;
            while jj+1 <= length(tmp_timea) && tmp_timea(jj+1)-tmp_timea(j) <= time_threshold
                jj = jj+1;
            end
            if k == 0 || jj ~= prev_jj
                if rand <= accept_ratio
                    if k ~= 0 || (k == 0 && segment_length(1) == 0)
                        cur_segment{length(target_y_i)+outlier_step_idx, 1} = feature_a(j:jj, :);
                        cur_segment{length(target_y_i)+outlier_step_idx, 2} = 6;
                        outlier_step_idx = outlier_step_idx+1;
                    end
                end
            end
            if k == 0
                next_j = jj+1;
            end
            if jj == prev_jj
                break;
            end
        end
        j = next_j;
    end
    
    % extract features from each segment
    [cur_feature, cur_label] = csegment2feature1_sep_sampling_single(cur_segment, bins, nfeatures);
    clear cur_segment;
    % resampling
    cur_target_i = cur_label~=6;

    % resampling the fault events for Random Forest
    nrep = round( length(find(cur_label~=6))*4/length(find(cur_label==6)) );
    train_feature_cell{i} = [cur_feature; repmat(cur_feature(cur_target_i, :), nrep, 1)];
    train_label_cell{i} = [cur_label; repmat(cur_label(cur_target_i, :), nrep, 1)];
    % train end

    %% construct testing data
    tmp_timea = tmp_timea_origin;
    feature_a = feature_a_origin;

    time_c1 = readtime(datac{1});
    time_c1 = time_c1(fault_index_removed:end);
    time_c2 = readtime(datac{2});
    time_c2 = time_c2(fault_index_removed:end);
    y_c = datac{3};
    y_c = y_c(fault_index_removed:end);
    time_c = [time_c1, time_c2];
    
    index_preserved = find(y_c ~= 6);
    test_time_c{i} = time_c(index_preserved, :);
    test_y_c{i} = y_c(index_preserved);

    dataa_preserved = tmp_timea >= min(time_c(:, 1));
    tmp_timea = tmp_timea(dataa_preserved);
    feature_a = feature_a(dataa_preserved, :);
    
    test_dataseq_cell{i} = feature_a;
    test_timeseq_cell{i} = tmp_timea;
    % test end
    
    
end

end

%% convert time stamps to real numbers
function [times] = readtime(time_raw)
% 2 kinds of time formats
if ~isempty(strfind(time_raw{1}, '/'))
    times = datenum(time_raw, 'yyyy/mm/dd HH:MM');
else
    times = datenum(time_raw, 'yyyy-mm-dd HH:MM:SS');
end
end